from .base import Base


class Admin(Base):
    def __init__(self, api):
        super(Admin, self).__init__(api)
        self.api = api

    def settings(self):
        """

        :return:
        """
        path = "/admin/settings"
        r = self.api.GET(path)
        return r

    def stats(self):
        """

        :return:
        """
        path = "/admin/stats"
        r = self.api.GET(path)
        return r

    def create_user(self, user):
        """

        :param user:
        :return:
        """
        create_user_path = "/admin/users"
        r = self.api.POST(create_user_path, json=user)
        return r

    def change_user_password(self, user_id, password):
        """

        :param user_id:
        :param password:
        :return:
        """
        change_user_password_path = "/admin/users/%s/password" % user_id
        r = self.api.PUT(change_user_password_path, json={"password": password})
        return r

    def change_user_permissions(self, user_id, is_grafana_admin):
        """

        :param user_id:
        :param is_grafana_admin:
        :return:
        """
        change_user_permissions = "/admin/users/%s/permissions" % user_id
        r = self.api.PUT(
            change_user_permissions, json={"isGrafanaAdmin": is_grafana_admin}
        )
        return r

    def delete_user(self, user_id):
        """

        :param user_id:
        :return:
        """
        delete_user_path = "/admin/users/%s" % user_id
        r = self.api.DELETE(delete_user_path)
        return r

    def pause_all_alerts(self, pause):
        """

        :param pause:
        :return:
        """
        change_user_permissions = "/admin/pause-all-alerts"
        r = self.api.POST(change_user_permissions, json={"paused": pause})
        return r



import unittest

import requests_mock

from grafana_api.grafana_face import GrafanaFace


class OrganizationTestCase(unittest.TestCase):
    def setUp(self):
        self.cli = GrafanaFace(
            ("admin", "admin"), host="localhost", url_path_prefix="", protocol="http"
        )

    @requests_mock.Mocker()
    def test_delete_snapshot_by_key(self, m):
        m.delete('http://localhost/api/orgs/1/users/2', json={"message": "User removed from organization"})
        annotation = self.cli.organizations.organization_user_delete(organization_id=1, user_id=2)
        self.assertEqual(annotation['message'], "User removed from organization")

    @requests_mock.Mocker()
    def test_organization_preference_get(self, m):
        m.get(
            "http://localhost/api/org/preferences",
            json={"theme": "", "homeDashboardId": 0, "timezone": ""}
        )

        result = self.cli.organizations.organization_preference_get()
        self.assertEqual(result["homeDashboardId"], 0)

    @requests_mock.Mocker()
    def test_organization_preference_update(self, m):
        m.put(
            "http://localhost/api/org/preferences",
            json={"message": "Preferences updated"}
        )
        preference = self.cli.organizations.organization_preference_update(theme="", home_dashboard_id=0,
                                                                           timezone="utc")
        self.assertEqual(preference["message"], "Preferences updated")

    @requests_mock.Mocker()
    def test_organization_user_update(self, m):
        m.patch(
            "http://localhost/api/orgs/1/users/2",
            json={"message": "Organization user updated"}
        )
        preference = self.cli.organizations.organization_user_update(organization_id=1, user_id=2, user_role="Admin")
        self.assertEqual(preference["message"], "Organization user updated")

    @requests_mock.Mocker()
    def test_organization_user_add(self, m):
        m.post(
            "http://localhost/api/orgs/1/users",
            json={"message": "User added to organization"}
        )
        preference = self.cli.organizations.organization_user_add(organization_id=1, user={
            "loginOrEmail": "user",
            "role": "Viewer"
        })
        self.assertEqual(preference["message"], "User added to organization")

    @requests_mock.Mocker()
    def test_organization_user_list(self, m):
        m.get(
            "http://localhost/api/orgs/1/users",
            json=[
                {
                    "orgId": 1,
                    "userId": 1,
                    "email": "admin@mygraf.com",
                    "login": "admin",
                    "role": "Admin"
                }
            ]
        )
        users = self.cli.organizations.organization_user_list(organization_id=1)
        self.assertEqual(len(users), 1)

    @requests_mock.Mocker()
    def test_list_organization(self, m):
        m.get(
            "http://localhost/api/orgs",
            json=[
                {
                    "orgId": 1,
                    "userId": 1,
                    "email": "admin@mygraf.com",
                    "login": "admin",
                    "role": "Admin"
                }
            ]
        )
        users = self.cli.organizations.list_organization()
        self.assertEqual(len(users), 1)

    @requests_mock.Mocker()
    def test_get_current_organization(self, m):
        m.get(
            "http://localhost/api/org",
            json={
                "id": 1,
                "name": "Main Org."
            }
        )
        orgs = self.cli.organization.get_current_organization()
        self.assertEqual(orgs['name'], "Main Org.")

    @requests_mock.Mocker()
    def test_update_current_organization(self, m):
        m.put(
            "http://localhost/api/org",
            json={"message": "Organization updated"}
        )
        org = self.cli.organization.update_current_organization(organization={
            "name": "Main Org."
        })
        self.assertEqual(org['message'], "Organization updated")

    @requests_mock.Mocker()
    def test_update_organization(self, m):
        m.put(
            "http://localhost/api/orgs/1",
            json={"message": "Organization updated"}
        )
        preference = self.cli.organizations.update_organization(organization_id=1, organization={
            "name": "Main Org 2."
        })
        self.assertEqual(preference["message"], "Organization updated")

    @requests_mock.Mocker()
    def test_delete_organization(self, m):
        m.delete(
            "http://localhost/api/orgs/1",
            json={"message": "Organization deleted"}
        )
        preference = self.cli.organizations.delete_organization(organization_id=1)
        self.assertEqual(preference["message"], "Organization deleted")

    @requests_mock.Mocker()
    def test_create_organization(self, m):
        m.post(
            "http://localhost/api/orgs",
            json={
                "orgId": "1",
                "message": "Organization created"
            }
        )
        preference = self.cli.organization.create_organization(organization={
            "name": "New Org."
        })
        self.assertEqual(preference["message"], "Organization created")

    @requests_mock.Mocker()
    def test_delete_user_current_organization(self, m):
        m.delete(
            "http://localhost/api/org/users/1",
            json={"message": "User removed from organization"}
        )
        preference = self.cli.organization.delete_user_current_organization(user_id=1)
        self.assertEqual(preference["message"], "User removed from organization")

    @requests_mock.Mocker()
    def test_add_user_current_organization(self, m):
        m.post(
            "http://localhost/api/org/users",
            json={"message": "User added to organization"}
        )
        preference = self.cli.organization.add_user_current_organization({
            "role": "Admin",
            "loginOrEmail": "admin"
        })
        self.assertEqual(preference["message"], "User added to organization")

    @requests_mock.Mocker()
    def test_update_user_current_organization(self, m):
        m.patch(
            "http://localhost/api/org/users/1",
            json={"message": "Organization user updated"}
        )
        preference = self.cli.organization.update_user_current_organization(user_id=1, user={
            "role": "Viewer",
        })
        self.assertEqual(preference["message"], "Organization user updated")

    @requests_mock.Mocker()
    def test_get_current_organization_users(self, m):
        m.get(
            "http://localhost/api/org/users",
            json=[
                {
                    "orgId": 1,
                    "userId": 1,
                    "email": "admin@mygraf.com",
                    "login": "admin",
                    "role": "Admin"
                }
            ]
        )
        org = self.cli.organization.get_current_organization_users()
        self.assertEqual(len(org), 1)

    @requests_mock.Mocker()
    def test_find_organization(self, m):
        m.get(
            "http://localhost/api/orgs/name/Main",
            json=
            {
                "id": 1,
                "name": "Main Org.",
                "address": {
                    "address1": "",
                    "address2": "",
                    "city": "",
                    "zipCode": "",
                    "state": "",
                    "country": ""
                }
            }
        )
        org = self.cli.organization.find_organization(org_name="Main")
        self.assertEqual(org['id'], 1)

    @requests_mock.Mocker()
    def test_switch_organization(self, m):
        m.post(
            "http://localhost/api/user/using/2",
            json={"message":"Active organization changed"}
        )
        preference = self.cli.organizations.switch_organization(organization_id=2)
        self.assertEqual(preference["message"], "Active organization changed")





from .base import Base


class Search(Base):
    def __init__(self, api):
        super(Search, self).__init__(api)
        self.api = api

    def search_dashboards(
        self,
        query=None,
        tag=None,
        type_=None,
        dashboard_ids=None,
        folder_ids=None,
        starred=None,
        limit=None,
    ):
        """

        :param query:
        :param tag:
        :param type_:
        :param dashboard_ids:
        :param folder_ids:
        :param starred:
        :param limit:
        :return:
        """
        list_dashboard_path = "/search"
        params = []

        if query:
            params.append("query=%s" % query)

        if tag:
            params.append("tag=%s" % tag)

        if type_:
            params.append("type=%s" % type_)

        if dashboard_ids:
            params.append("dashboardIds=%s" % dashboard_ids)

        if folder_ids:
            params.append("folderIds=%s" % folder_ids)

        if starred:
            params.append("starred=%s" % starred)

        if limit:
            params.append("limit=%s" % limit)

        list_dashboard_path += "?"
        list_dashboard_path += "&".join(params)

        r = self.api.GET(list_dashboard_path)
        return r



from .base import Base


class Dashboard(Base):
    def __init__(self, api):
        super(Dashboard, self).__init__(api)
        self.api = api

    def get_dashboard(self, dashboard_uid):
        """

        :param dashboard_uid:
        :return:
        """
        get_dashboard_path = "/dashboards/uid/%s" % dashboard_uid
        r = self.api.GET(get_dashboard_path)
        return r

    def update_dashboard(self, dashboard):
        """

        :param dashboard:
        :return:
        """
        put_dashboard_path = "/dashboards/db"
        r = self.api.POST(put_dashboard_path, json=dashboard)
        return r

    def delete_dashboard(self, dashboard_uid):
        """

        :param dashboard_uid:
        :return:
        """
        delete_dashboard_path = "/dashboards/uid/%s" % dashboard_uid
        r = self.api.DELETE(delete_dashboard_path)
        return r

    def get_home_dashboard(self):
        """

        :return:
        """
        get_home_dashboard_path = "/dashboards/home"
        r = self.api.GET(get_home_dashboard_path)
        return r

    def get_dashboards_tags(self):
        """

        :return:
        """
        get_dashboards_tags_path = "/dashboards/tags"
        r = self.api.GET(get_dashboards_tags_path)
        return r

    def get_dashboard_permissions(self, dashboard_id):
        """

        :param dashboard_id:
        :return:
        """
        get_dashboard_permissions_path = "/dashboards/id/%s/permissions" % dashboard_id
        r = self.api.GET(get_dashboard_permissions_path)
        return r

    def update_dashboard_permissions(self, dashboard_id, items):
        """

        :param dashboard_id:
        :param items:
        :return:
        """
        update_dashboard_permissions_path = (
            "/dashboards/id/%s/permissions" % dashboard_id
        )
        r = self.api.POST(update_dashboard_permissions_path, json=items)
        return r



import requests
import requests.auth


class GrafanaException(Exception):
    pass


class GrafanaServerError(Exception):
    """
    5xx
    """

    pass


class GrafanaClientError(Exception):
    """
    Invalid input (4xx errors)
    """

    pass


class GrafanaBadInputError(GrafanaClientError):
    """
    400
    """

    pass


class GrafanaUnauthorizedError(GrafanaClientError):
    """
    401
    """

    pass


class TokenAuth(requests.auth.AuthBase):
    def __init__(self, token):
        self.token = token

    def __call__(self, request):
        request.headers.update({"Authorization": "Bearer {0}".format(self.token)})
        return request


class GrafanaAPI:
    def __init__(
        self,
        auth,
        host="localhost",
        port=None,
        url_path_prefix="",
        protocol="http",
        verify=True,
        timeout=5.0,
    ):
        self.auth = auth
        self.verify = verify
        self.timeout = timeout
        self.url_host = host
        self.url_port = port
        self.url_path_prefix = url_path_prefix
        self.url_protocol = protocol

        def construct_api_url():
            params = {
                "protocol": self.url_protocol,
                "host": self.url_host,
                "url_path_prefix": self.url_path_prefix,
            }

            if self.url_port is None:
                url_pattern = "{protocol}://{host}/{url_path_prefix}api"
            else:
                params["port"] = self.url_port
                url_pattern = "{protocol}://{host}:{port}/{url_path_prefix}api"

            return url_pattern.format(**params)

        self.url = construct_api_url()

        self.s = requests.Session()
        if not isinstance(self.auth, tuple):
            self.auth = TokenAuth(self.auth)
        else:
            self.auth = requests.auth.HTTPBasicAuth(*self.auth)

    def __getattr__(self, item):
        def __request_runnner(url, json=None, headers=None):
            __url = "%s%s" % (self.url, url)
            runner = getattr(self.s, item.lower())
            r = runner(
                __url, json=json, headers=headers, auth=self.auth, verify=self.verify, timeout=self.timeout
            )
            if 500 <= r.status_code < 600:
                raise GrafanaServerError(
                    "Client Error {0}: {1}".format(r.status_code, r.json()['message'])
                )
            elif r.status_code == 400:
                raise GrafanaBadInputError("Bad Input: `{0}`".format(r.text))
            elif r.status_code == 401:
                raise GrafanaUnauthorizedError("Unauthorized")
            elif 400 <= r.status_code < 500:
                raise GrafanaClientError(
                    "Client Error {0}: {1}".format(r.status_code, r.text)
                )
            return r.json()
        return __request_runnner



from setuptools import setup, find_packages
from subprocess import check_output


def get_version():
    try:
        tag = check_output(
            ["git", "describe", "--tags", "--abbrev=0", "--match=[0-9]*"]
        )
        return tag.decode("utf-8").strip("\n")
    except Exception:
        raise RuntimeError(
            "The version number cannot be extracted from git tag in this source "
            "distribution; please either download the source from PyPI, or check out "
            "from GitHub and make sure that the git CLI is available."
        )


with open("README.md") as file:
    long_description = file.read()

setup(
    name="grafana_api",
    version=get_version(),
    description="Yet another Python library for Grafana API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/m0nhawk/grafana_api",
    author="Andrew Prokhorenkov",
    author_email="andrew.prokhorenkov@gmail.com",
    license="MIT",
    packages=find_packages(),
    install_requires=["pyyaml", "requests"],
    tests_require=["unittest-xml-reporting", "requests-mock"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Internet",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    zip_safe=False,
)



import unittest

import requests_mock

from grafana_api.grafana_face import GrafanaFace


class SnapshotTestCase(unittest.TestCase):
    def setUp(self):
        self.cli = GrafanaFace(
            ("admin", "admin"), host="localhost", url_path_prefix="", protocol="http"
        )

    @requests_mock.Mocker()
    def test_create_new_snapshot(self, m):
        m.post(
            "http://localhost/api/snapshots",
            json={
                "deleteKey": "XXXXXXX",
                "deleteUrl": "myurl/api/snapshots.py-delete/XXXXXXX",
                "key": "YYYYYYY",
                "url": "myurl/dashboard/snapshot/YYYYYYY"
            },
        )
        snapshot = self.cli.snapshots.create_new_snapshot(dashboard={
            "editable": "false",
            "hideControls": "true",
            "nav": [
                {
                    "enable": "false",
                    "type": "timepicker"
                }
            ],
            "rows": [
                {

                }
            ],
            "style": "dark",
            "tags": [],
            "templating": {
                "list": [
                ]
            },
            "time": {
            },
            "timezone": "browser",
            "title": "Home",
            "version": 5
        }, name="Test", key="YYYYYYY", delete_key="XXXXXXX", external=True, expires=3600)
        self.assertEqual(snapshot["key"], "YYYYYYY")

    @requests_mock.Mocker()
    def test_create_new_snapshot_without_optional(self, m):
        m.post(
            "http://localhost/api/snapshots",
            json={
                "deleteKey": "XXXXXXX",
                "deleteUrl": "myurl/api/snapshots.py-delete/XXXXXXX",
                "key": "YYYYYYY",
                "url": "myurl/dashboard/snapshot/YYYYYYY"
            },
        )
        snapshot = self.cli.snapshots.create_new_snapshot(dashboard={
            "editable": "false",
            "hideControls": "true",
            "nav": [
                {
                    "enable": "false",
                    "type": "timepicker"
                }
            ],
            "rows": [
                {

                }
            ],
            "style": "dark",
            "tags": [],
            "templating": {
                "list": [
                ]
            },
            "time": {
            },
            "timezone": "browser",
            "title": "Home",
            "version": 5
        })
        self.assertEqual(snapshot["key"], "YYYYYYY")

    @requests_mock.Mocker()
    def test_get_dashboard_snapshots(self, m):
        m.get(
            "http://localhost/api/dashboard/snapshots",
            json=[
                {
                    "id": 8,
                    "name": "Home",
                    "key": "YYYYYYY",
                    "orgId": 1,
                    "userId": 1,
                    "external": False,
                    "externalUrl": "",
                    "expires": "2200-13-32T25:23:23+02:00",
                    "created": "2200-13-32T28:24:23+02:00",
                    "updated": "2200-13-32T28:24:23+02:00"
                }
            ]
        )
        dashboards = self.cli.snapshots.get_dashboard_snapshots()
        self.assertEqual(len(dashboards), 1)

    @requests_mock.Mocker()
    def test_get_snapshot_by_key(self, m):
        m.get(
            "http://localhost/api/snapshots/YYYYYYY",
            json=[
                {
                    "id": 8,
                    "name": "Home",
                    "key": "YYYYYYY",
                    "orgId": 1,
                    "userId": 1,
                    "external": False,
                    "externalUrl": "",
                    "expires": "2200-13-32T25:23:23+02:00",
                    "created": "2200-13-32T28:24:23+02:00",
                    "updated": "2200-13-32T28:24:23+02:00"
                }
            ]
        )
        dashboards = self.cli.snapshots.get_snapshot_by_key(key="YYYYYYY")
        self.assertEqual(len(dashboards), 1)

    @requests_mock.Mocker()
    def test_delete_snapshot_by_key(self, m):
        m.delete('http://localhost/api/snapshots/YYYYYYY', json={"message": "Snapshot deleted. It might take an hour "
                                                                            "before it's cleared from any CDN "
                                                                            "caches."})
        annotation = self.cli.snapshots.delete_snapshot_by_key(snapshot_id="YYYYYYY")
        self.assertEqual(annotation['message'], "Snapshot deleted. It might take an hour before it's cleared from any "
                                                "CDN caches.")

    @requests_mock.Mocker()
    def test_delete_snapshot_by_delete_key(self, m):
        m.get('http://localhost/api/snapshots-delete/XXXXXXX', json={"message": "Snapshot deleted. It might take an hour "
                                                                            "before it's cleared from any CDN "
                                                                            "caches."})
        annotation = self.cli.snapshots.delete_snapshot_by_delete_key(snapshot_delete_key="XXXXXXX")
        self.assertEqual(annotation['message'], "Snapshot deleted. It might take an hour before it's cleared from any "
                                                "CDN caches.")

